//=================================================
//
// Class			StreamController
//
// Author:			Grzegorz Lewandowski
// created			01/14/98
//
//-------------------------------------------------

import java.io.*;

public class StreamController extends InputStream
{
	private int state = NEW;
	private InputStream stream = null;
	//---------------------------------------------
	static final int NEW = 0;
	static final int OPEN = 1;
	static final int CLOSED = 2;
	static final int EOF = 3;

	public StreamController( InputStream s )
	{
		super();
		stream = s;
		state = OPEN;
	}

	public synchronized int read() throws IOException
	{
		while( state == CLOSED )
		{
			try
			{
				wait();
			}
			catch( Exception e )
			{
				e.printStackTrace( System.err );
			}
		}

		if( state == EOF )
		{
			System.out.println( "StreamController: substream has ended." );
			return -1;
		}

		if( state == OPEN )
		{
			return stream.read();
		}

		System.err.println( "StreamController: wrong state." );
		return -1;
	}

	public synchronized int read( byte  b[], int  off, int  len ) throws IOException
	{
		while( state == CLOSED )
		{
			try
			{
				wait();
			}
			catch( Exception e )
			{
				e.printStackTrace( System.err );
			}
		}

		if( state == EOF )
		{
			System.out.println( "StreamController: substream has ended." );
			return -1;
		}

		if( state == OPEN )
		{
			return stream.read( b, off, len );
		}

		System.err.println( "StreamController: wrong state." );
		return -1;
	}

	public synchronized void open()
	{
		state = OPEN;
		notify();
		System.out.println( "StreamController: substream open." );
	}

	public synchronized void shut()
	{
		state = CLOSED;
		notify();
		System.out.println( "StreamController: substream shut." );
	}

	public synchronized void eof()
	{
		state = EOF;
		notify();
		System.out.println( "StreamController: substream eof." );
	}

}